"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getModelsQueue = void 0;
const { RedisQueueClient } = require("redis-ordered-queue");
const Redis = require("ioredis");
const defaultRedisKeyPrefix = "mongoose-sort-encrypted-field";
const { REDIS_QUEUE_CLIENT_OPTIONS } = require("./constants");
const { updateSortFieldsForDocument, generateSortIdForAllDocuments } = require("./utils");
let modelsQueue;
class ModelsQueue {
    constructor(redisQueueClientOptions) {
        this.groupIdToModelMap = {};
        redisQueueClientOptions = Object.assign(Object.assign({}, REDIS_QUEUE_CLIENT_OPTIONS), redisQueueClientOptions);
        const { redis, batchSize, groupVisibilityTimeoutMs, pollingTimeoutMs, consumerCount, redisKeyPrefix } = redisQueueClientOptions;
        let redisClient = redis;
        if (!(redis instanceof Redis)) {
            redisClient = new Redis(redis);
        }
        this.client = new RedisQueueClient({
            redis: redisClient,
            batchSize,
            groupVisibilityTimeoutMs,
            pollingTimeoutMs,
            consumerCount,
            redisKeyPrefix,
        });
        this.client.startConsumers({ handleMessage: this.handleMessage });
    }
    handleMessage({ data, context: { lock: { groupId }, }, }) {
        return __awaiter(this, void 0, void 0, function* () {
            data.model = modelsQueue.groupIdToModelMap[groupId];
            if (!data.model.schema.options.sortEncryptedFieldsOptions.silent) {
                const noOfPendingJobs = (yield modelsQueue.client.getMetrics(100)).topMessageGroupsMessageBacklogLength;
                console.log(`mongoose-sort-encrypted-field -> handleMessage() -> noOfPendingJobs: ${noOfPendingJobs}`);
            }
            if (data.generateSortIdForAllDocuments) {
                yield generateSortIdForAllDocuments(data);
                return;
            }
            yield updateSortFieldsForDocument(data);
        });
    }
    addJob(groupId, data) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.client.send({ groupId, data });
        });
    }
    registerGroup(groupId, model) {
        if (!this.groupIdToModelMap[groupId]) {
            this.groupIdToModelMap[groupId] = model;
        }
    }
    removeAllJobs(modelName) {
        return __awaiter(this, void 0, void 0, function* () {
            const keys = yield this.client.redis.keys(`${defaultRedisKeyPrefix}::msg-group-queue::${modelName}`);
            var pipeline = this.client.redis.pipeline();
            keys.forEach(function (key) {
                pipeline.del(key);
            });
            pipeline.exec();
        });
    }
}
function getModelsQueue(redisOptions) {
    if (!modelsQueue) {
        modelsQueue = new ModelsQueue(redisOptions);
    }
    return modelsQueue;
}
exports.getModelsQueue = getModelsQueue;
